import unittest
from nom_de_votre_package import module  # Ajustez le chemin d'importation selon votre structure

class TestModule(unittest.TestCase):
    def test_fonction1(self):
        # Supposons que module.fonction1() doive retourner True
        self.assertTrue(module.fonction1())

    def test_fonction2_avec_parametre(self):
        # Supposons que module.fonction2(5) doive retourner 10
        resultat_attendu = 10
        self.assertEqual(module.fonction2(5), resultat_attendu)

# Ceci permet d'exécuter les tests si ce fichier est exécuté directement
if __name__ == '__main__':
    unittest.main()


#coding: utf-8
import threading
class Overflow(Exception):
    pass

class Underflow(Exception):
    pass

class RingBuffer:
    def __init__(self, buf):
        self._buf = buf
        self._r = 0
        self._size = 0

    def __len__(self):
        return len(self._buf)

    @property
    def read_size(self):
        return self._size

    @property
    def write_size(self):
        return len(self) - self.read_size

    def read_only(self, buf):
        size = len(buf)
        if size == 0:
            return
        if size > self.read_size:
            raise Underflow
        f = self._r
        l = (f + size) % len(self)
        if f < l:
            buf[:] = self._buf[f:l]
        else:
            n = len(self) - f
            buf[:n] = self._buf[f:]
            buf[n:] = self._buf[:l]

    def remove_only(self, size):
        if size < 0:
            raise ValueError("'size' must be a non-negative number")
        if size > self.read_size:
            raise Underflow
        self._r = (self._r + size) % len(self)
        self._size -= size

    def read(self, buf):
        self.read_only(buf)
        self.remove_only(len(buf))

    def write(self, buf):
        size = len(buf)
        if size == 0:
            return
        if size > self.write_size:
            raise Overflow
        f = (self._r + self._size) % len(self)
        l = (f + size) % len(self)
        if f < l:
            self._buf[f:l] = buf
        else:
            n = len(self) - f
            self._buf[f:] = buf[:n]
            self._buf[:l] = buf[n:]
        self._size += size

class ConcurrentRingBuffer:
    def __init__(self, buf):
        self._rb = RingBuffer(buf)
        self._lock = threading.Lock()
        self._overflow = threading.Condition(self._lock)
        self._underflow = threading.Condition(self._lock)

    def write(self, buf, block=True, timeout=None):
        if len(buf) > len(self._rb):
            raise ValueError("'buf' is too big")
        with self._lock:
            if block and not self._overflow.wait_for(lambda: len(buf) <= self._rb.write_size, timeout):
                raise Overflow
            self._rb.write(buf)
            self._underflow.notify()

    def read(self, buf, remove_size=None, block=True, timeout=None):
        if len(buf) > len(self._rb):
            raise ValueError("'buf' is too big")
        if remove_size is not None and (remove_size < 0 or remove_size > len(buf)):
            raise ValueError("'remove_size' must be non-negative and not exceed 'len(buf)'")
        with self._lock:
            if block and not self._underflow.wait_for(lambda: len(buf) <= self._rb.read_size, timeout):
                raise Underflow
            self._rb.read_only(buf)
            self._rb.remove_only(len(buf) if remove_size is None else remove_size)
            self._overflow.notify()


# coding: utf-8
import pkg_resources

class Files_WordRecognition_tflite:
    model = pkg_resources.resource_filename(__name__, 'models/star_trek_activation_phrase_v2.tflite')
    labels_file = pkg_resources.resource_filename(__name__, 'labels/star_trek_activation_phrase_v2.txt')

class labels_activation_phrase:
    label_0 = "Background Noise"
    label_1 = "annuler"
    label_2 = "ordinateur"
    # ##############################
    min_score_label_1 = float(0.3)
    min_score_label_2 = float(0.3)
    min_score_label_3 = float(0.3)



#coding: utf-8

import queue
import sys
import contextlib
import json
import numpy as np
import threading
import re
import pyaudio
from . import ring_buffer
from tflite_support import metadata
import tflite_runtime.interpreter as tflite
import librosa
from multiprocessing import Process
from .activation_defaults import Files_WordRecognition_tflite, labels_activation_phrase


def _associated_labels_file(metadata_json):
    for ot in metadata_json['subgraph_metadata'][0]['output_tensor_metadata']:
        if 'associated_files' in ot:
            for af in ot['associated_files']:
                if af['type'] in ('TENSOR_AXIS_LABELS', 'TENSOR_VALUE_LABELS'):
                    return af['name']
    raise ValueError('Model metadata does not have associated labels file')


def read_labels_from_metadata(model):
    """Read labels from the model file metadata.

    Args:
        model (str): Path to the ``.tflite`` file.
    Returns:
        A dictionary of (int, string), mapping label ids to text labels.
    """
    displayer = metadata.MetadataDisplayer.with_model_file(model)
    metadata_json = json.loads(displayer.get_metadata_json())
    labels_file = _associated_labels_file(metadata_json)
    labels = displayer.get_associated_file_buffer(labels_file).decode()
    return {i: label for i, label in enumerate(labels.splitlines())}


def read_labels_file(filepath):
    """ 
    Lire les labels depuis un fichier texte et retourner cela comme un dictionnaire.
    Arguments:
    - filepath (str): Le chemin vers le fichier de labels.
    Sortie:
    - dictionnaire de (int, str): Les labels avec les ids comme clés et les descriptions comme valeurs.
    """
    with open(filepath, 'r', encoding='utf-8') as f:
        lines = f.readlines()
    ret = {}
    for num_row, content in enumerate(lines):
        pair = re.split(r'[:\s]+', content.strip(), maxsplit=1)
        if len(pair) == 2 and pair[0].strip().isdigit():
            ret[int(pair[0])] = pair[1].strip()
        else:
            ret[num_row] = content.strip()
    return ret 


@contextlib.contextmanager
def pyaudio_stream(*args, **kwargs):
    """Context manager for the PyAudio stream."""
    audio = pyaudio.PyAudio()
    try:
        stream = audio.open(*args, **kwargs)
        try:
            yield stream
        finally:
            stream.stop_stream()
            stream.close()
    finally:
        audio.terminate()


def model_audio_properties(model_file):
    """
    Returns the audio sample rate and number of channels that must be used with
    the given model (as a tuple in that order).
    """
    displayer = metadata.MetadataDisplayer.with_model_file(model_file)
    metadata_json = json.loads(displayer.get_metadata_json())
    if metadata_json['name'] != 'AudioClassifier':
        raise ValueError('Model must be an audio classifier')
    props = metadata_json['subgraph_metadata'][0]['input_tensor_metadata'][0]['content']['content_properties']
    return int(props['sample_rate']), int(props['channels'])


def activation_callback(label, score):
    print(label, '=>', score)
    print('Activation detected')
    exit(0)


def activation_phrase_detection(label, score):
    if label == labels_activation_phrase.label_2:
        print(f"{label} is detected...")
        if score > labels_activation_phrase.min_score_label_2:
            print(f"ok now the label '{label}' pass with the score of : '{score}'")
            return True
    return False


class AudioClassifier:
    """Performs classifications with a speech classification model.

    Args:
        model (str): Path to a ``.tflite`` file.
        labels_file (str): Path to a labels file (required only if the model
            does not include metadata labels). If provided, this overrides the
            labels file provided in the model metadata.
        inference_overlap_ratio (float): The amount of audio that should overlap
            between each sample used for inference. May be 0.0 up to (but not
            including) 1.0. For example, if set to 0.5 and the model takes a
            one-second sample as input, the model will run an inference every
            half second, or if set to 0, it will run once each second.
        buffer_size_secs (float): The length of audio to hold in the audio
            buffer.
        buffer_write_size_secs (float): The length of audio to capture into the
            buffer with each sampling from the microphone.
        audio_device_index (int): The audio input device index to use.
    """
    def __init__(self, callback_start_assistant, model, labels_file=None, inference_overlap_ratio=0.1,
                 buffer_size_secs=2.0, buffer_write_size_secs=0.1, audio_device_index=None, **kwargs):
        self._queue = queue.Queue()
        self._stop_listening = threading.Event()
        self._thread = threading.Thread(
            target=self.classify_audio,
            kwargs={'callback': self.handle_results, 'model': model, 'labels_file': labels_file,
                    'inference_overlap_ratio': inference_overlap_ratio, 'buffer_size_secs': buffer_size_secs,
                    'buffer_write_size_secs': buffer_write_size_secs, 'audio_device_index': audio_device_index, **kwargs},
            daemon=True)
        self._thread.start()
        self._process_assistant = None
        self.callback_start_assistant = callback_start_assistant
        self._audio_stream = None
        self.model = model
        self.labels_file = labels_file
        self.inference_overlap_ratio = inference_overlap_ratio
        self.buffer_size_secs = buffer_size_secs
        self.buffer_write_size_secs = buffer_write_size_secs
        self.audio_device_index = audio_device_index
        self.kwargs = kwargs

    def classify_audio(self, model, callback, labels_file=None, inference_overlap_ratio=0.1,
                       buffer_size_secs=2.0, buffer_write_size_secs=0.1, audio_device_index=None):
        """
        Continuously classifies audio samples from the microphone, yielding results
        to your own callback function.
        """
        sample_rate_hz, channels = model_audio_properties(model)

        if labels_file is not None:
            labels = read_labels_file(labels_file)
        else:
            labels = read_labels_from_metadata(model)

        if not model:
            raise ValueError('model must be specified')

        if buffer_size_secs <= 0.0:
            raise ValueError('buffer_size_secs must be positive')

        if buffer_write_size_secs <= 0.0:
            raise ValueError('buffer_write_size_secs must be positive')

        if inference_overlap_ratio < 0.0 or inference_overlap_ratio >= 1.0:
            raise ValueError('inference_overlap_ratio must be in [0.0 .. 1.0)')

        interpreter = tflite.Interpreter(model_path=model)
        interpreter.allocate_tensors()

        input_details = interpreter.get_input_details()
        waveform_input_index = input_details[0]['index']
        _, num_audio_frames = input_details[0]['shape']
        waveform = np.zeros(num_audio_frames, dtype=np.float32)

        output_details = interpreter.get_output_details()
        scores_output_index = output_details[0]['index']

        def preprocess_audio(audio_data, sr=16000):
            mel_spectrogram = librosa.feature.melspectrogram(y=audio_data, sr=sr, n_mels=128)
            log_mel_spectrogram = librosa.power_to_db(mel_spectrogram)
            return log_mel_spectrogram.T

        ring_buffer_size = int(buffer_size_secs * sample_rate_hz)
        frames_per_buffer = int(buffer_write_size_secs * sample_rate_hz)
        remove_size = int((1.0 - inference_overlap_ratio) * len(waveform))

        rb = ring_buffer.ConcurrentRingBuffer(
            np.zeros(ring_buffer_size, dtype=np.float32))

        def stream_callback(in_data, frame_count, time_info, status):
            try:
                rb.write(np.frombuffer(in_data, dtype=np.float32), block=False)
            except ring_buffer.Overflow:
                print('WARNING: Dropping input audio buffer', file=sys.stderr)
            return None, pyaudio.paContinue

        with pyaudio_stream(format=pyaudio.paFloat32,
                            channels=channels,
                            rate=sample_rate_hz,
                            frames_per_buffer=frames_per_buffer,
                            stream_callback=stream_callback,
                            input=True,
                            input_device_index=audio_device_index) as stream:
            self._audio_stream = stream
            keep_listening = True
            while keep_listening:
                while not self._stop_listening.is_set():
                    rb.read(waveform, remove_size=remove_size)
                    interpreter.set_tensor(waveform_input_index, [waveform])
                    interpreter.invoke()
                    scores = interpreter.get_tensor(scores_output_index)
                    scores = np.mean(scores, axis=0)
                    prediction = np.argmax(scores)
                    keep_listening = callback(labels[prediction], scores[prediction])

    def close_audio_stream(self):
        if self._audio_stream is not None:
            self._audio_stream.stop_stream()
            self._audio_stream.close()
            self._audio_stream = None

    def stop_listening(self):
        """Méthode pour arrêter l'écoute."""
        self._stop_listening.set()
        self.close_audio_stream()

    def resume_listening(self):
        """Méthode pour reprendre l'écoute."""
        self._stop_listening.clear()
        self._thread = threading.Thread(
            target=self.classify_audio,
            kwargs={'callback': self.handle_results, 'model': self.model, 'labels_file': self.labels_file,
                    'inference_overlap_ratio': self.inference_overlap_ratio, 'buffer_size_secs': self.buffer_size_secs,
                    'buffer_write_size_secs': self.buffer_write_size_secs, 'audio_device_index': self.audio_device_index, **self.kwargs},
            daemon=True)
        self._thread.start()

    def handle_results(self, label, score):
        label_, score_ = str(label), float(score)
        print('CALLBACK: ', label_, '=>', score_)
        print(f"{type(label_)} => {type(score_)}")

        activation_detected = activation_phrase_detection(label_, score_)
        print(activation_detected)
        if activation_detected:
            if self._process_assistant is None or not self._process_assistant.is_alive():
                self.stop_listening()
                self._process_assistant = Process(target=self.callback_start_assistant, args=())
                self._process_assistant.start()
                self._process_assistant.join()
                self.resume_listening()
            return False 
        return True

    def next(self, block=True):
        """
        Returns a single speech classification.

        Each time you call this, it pulls from a queue of recent
        classifications. So even if there are many classifications in a short
        period of time, this always returns them in the order received.

        Args:
            block (bool): Whether this function should block until the next
                classification arrives (if there are no queued classifications).
                If False, it always returns immediately and returns None if the
                classification queue is empty.
        """
        try:
            result = self._queue.get(block)
            self._queue.task_done()
            return result
        except queue.Empty:
            return None



#coding: utf-8

from activation_defaults import Files_WordRecognition_tflite, labels_activation_phrase
from audio import AudioClassifier
from time import sleep


def activation_callback():
    print('Activation detected')
    sleep(20)
    print('Bye Bye !')
    exit(0)


def run():
    MODEL_FILE = Files_WordRecognition_tflite.model
    LABELS_FILE = Files_WordRecognition_tflite.labels_file

    classifier = AudioClassifier(callback_start_assistant=activation_callback, model=MODEL_FILE, labels_file=LABELS_FILE)

    while True:
        result = classifier.next(block=True)
        if result is not None:
            label, score = result
            print('Classification:', label, 'score:', score)

def main():
    try:
        run()
    except KeyboardInterrupt:
        print("Interruption par l'utilisateur")
        exit(0)
    except Exception as e:
        print(e)
        exit(1)

if __name__ == '__main__':
    main()




from setuptools import setup, find_packages

setup(
    name='word_activation_recognition',
    version='0.1.1',
    packages=find_packages(),
    install_requires=[
        'tflite-runtime',
        'tflite-support',
        'pandas',
        'pyaudio',
        'numpy<2.0.0',
        'librosa',
        'sounddevice',
        'soundfile',
    ],
    package_data={
        'word_activation_recognition': ['models/*.tflite', 'labels/*.txt'],
    },
    entry_points={
        'console_scripts': [
            'word_activation_recognition=word_activation_recognition.classify_audio:main',
        ],
    },
    author='0x07cb',
    author_email='83157348+0x07CB@users.noreply.github.com',
    license='MIT',
    description='A library for audio keyword detection and processing with TensorFlow Lite models',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/smartestmanuniverse/word_activation_recognition',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.11',
)






#!/usr/bin/env python3
#coding: utf-8

#from aiymakerkit import audio

import numpy as np
import pyaudio
import sounddevice as sd
import librosa
import tflite_runtime.interpreter as tflite

class labels_activation_phrase:
    label_0 = "0 Background Noise"
    label_1 = "1 annuler"
    label_2 = "2 ordinateur"
    # ##############################
    min_score_label_2 = float(0.3)
    min_score_label_1 = float(0.3)
    min_score_label_3 = float(0.3)

def activation_callback(label, score):
    print(label, '=>', score)
    print('Activation detected')
    exit(0)

def activation_phrase_detection(label, score):
    if label == labels_activation_phrase.label_2:
        print(f"{label} is detected...")
        if score > labels_activation_phrase.min_score_label_2:
            print(f"ok now the label '{label}' pass with the score of : '{score}'")
            return True
        else:
            return False
    else:
        return False
    return False

def choice_bin_response_detection(label, score):
    if label == labels_activation_phrase.label_1:
        if score > labels_activation_phrase.min_score_label_1:
            return True
        else:
            return False
    elif label == labels_activation_phrase.label_3:
        if score > labels_activation_phrase.min_score_label_3:
            return True
        else:
            return False
    else:
        return False
    return False

def audio_activation(activation_callback):
    def handle_results(label_, score_):
        label, score = str(label_), float(score_)
        print('CALLBACK: ', label, '=>', score)
        print(f"{type(label)} => {type(score)}")

        activation_detected = activation_phrase_detection(label, score)
        print(activation_detected)
        if activation_detected:
            activation_callback(label, score)
            return False 
        return True

    model_file = 'models/star_trek_activation_phrase_v2.tflite'

    audio.classify_audio(
                            model=model_file, 
                            callback=handle_results
                        )


# ##############################
# 
# ##############################
# def preprocess_audio(audio_path):
#    # Charger l'audio et convertir en Mel spectrogram (à adapter selon ton besoin)
#    mel_spectrogram = generate_mel_spectrogram(audio_path) # Fonction personnalisée
#    input_data = np.array(mel_spectrogram, dtype=np.float32)
#    return input_data


def generate_mel_spectrogram(audio_path):
    y, sr = librosa.load(audio_path, sr=16000)
    mel_spectrogram = librosa.feature.melspectrogram(y, sr=sr, n_mels=128)
    log_mel_spectrogram = librosa.power_to_db(mel_spectrogram)
    return log_mel_spectrogram.T




# ##############################
def main():
    model_file = 'models/star_trek_activation_phrase_v2.tflite'

    # Charger le modèle
    interpreter = tflite.Interpreter(model_path=model_file)
    interpreter.allocate_tensors()

    # Obtenir les détails des entrées et sorties
    input_details = interpreter.get_input_details()
    output_details = interpreter.get_output_details()

    # Paramètres du flux audio
    sample_rate = 16000  # Fréquence d'échantillonnage
    block_size = 1024  # Taille des blocs d'échantillons


    # Fonction de prétraitement pour convertir les données audio en Mel spectrogram
    def preprocess_audio(audio_data, sr=16000):
        nonlocal interpreter, input_details
        mel_spectrogram = librosa.feature.melspectrogram(y=audio_data, sr=sr, n_mels=128)
        log_mel_spectrogram = librosa.power_to_db(mel_spectrogram)
        return log_mel_spectrogram.T

    # Fonction pour traiter le flux audio en temps réel
    def audio_callback(indata, frames, time, status):
        nonlocal interpreter, input_details, output_details
        if status:
            print(status, flush=True)
        # Prétraitement de l'audio
        mel_spectrogram = preprocess_audio(indata[:, 0])
        input_data = np.expand_dims(mel_spectrogram, axis=0).astype(np.float32)

        # Faire une prédiction avec le modèle TFLite
        interpreter.set_tensor(input_details[0]['index'], input_data)
        interpreter.invoke()
        output_data = interpreter.get_tensor(output_details[0]['index'])
        
        # Afficher le résultat de la prédiction
        print("Résultat de la prédiction:", output_data, flush=True)
 
    try:
        # Démarrer le flux audio avec SoundDevice
        with sd.InputStream(callback=audio_callback, channels=1, samplerate=sample_rate, blocksize=block_size):
            print("En écoute... Appuyez sur Ctrl+C pour arrêter.", flush=True)
            while True:
                pass
    except KeyboardInterrupt:
        print("Interruption par l'utilisateur")
        exit(0)
    
    



if __name__ == '__main__':
    try:
        main()
        # audio_activation(activation_callback=activation_callback)
    except KeyboardInterrupt:
        print('Bye')
        exit(0)
    except Exception as e:
        print('ERROR:', e)
        exit(1)
    finally:
        print('Bye')
        exit(0)

